#include <assert.h>
#include "pixelbuf.h"

bool pixelbuf_init(PixelBuf* pixelbuf, size_t width, size_t height) {
   size_t pixelbuf_size = sizeof(guchar) * 3 * width * height;
   guchar* buf = malloc(pixelbuf_size);
   if (!buf) return false;
   pixelbuf->width = width;
   pixelbuf->height = height;
   pixelbuf->pixelbuf = buf;
   return true;
}

void pixelbuf_set(PixelBuf* pixelbuf, size_t row, size_t col, RGBColor color) {
   assert(row < pixelbuf->height && col < pixelbuf->width);
   guchar* buf = pixelbuf->pixelbuf + (row * pixelbuf->width + col) * 3;
   *buf++ = color.red * 255;
   *buf++ = color.green * 255;
   *buf++ = color.blue * 255;
}

bool pixelbuf_gen_jpeg(PixelBuf* pixelbuf, const char* filename) {
   GdkPixbuf* gdk_pixbuf = gdk_pixbuf_new_from_data(
      /* data = */ pixelbuf->pixelbuf,
      /* colorspace = */ GDK_COLORSPACE_RGB,
      /* has_alpha = */ FALSE,
      /* bits_per_sample = */ 8,
      /* width = */ pixelbuf->width,
      /* height = */ pixelbuf->height,
      /* rowstride = */ sizeof(guchar) * 3 * pixelbuf->width,
      /* destroy_fn = */ NULL,
      /* destroy_fn_data = */ NULL);
   if (!gdk_pixbuf) return false;
   bool ok = gdk_pixbuf_save(gdk_pixbuf, filename, "jpeg", (GError **)0,
      "quality", "100", (void*)0);
   g_object_unref(gdk_pixbuf);
   return ok;
}

void pixelbuf_free(PixelBuf* pixelbuf) {
   free(pixelbuf->pixelbuf);
   pixelbuf->pixelbuf = 0; pixelbuf->width = 0; pixelbuf->height = 0;
}
